@@ -1,4 +1,4 @@
-#  Copyright (c) 2023 SIMBA Chain Inc. https://simbachain.com
+#  Copyright (c) 2024 SIMBA Chain Inc. https://simbachain.com
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -17,25 +17,27 @@
 #  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 #  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 #  THE SOFTWARE.
-
+import asyncio
 import base64
 import logging
-import time
 
-from typing import AsyncGenerator, List, Optional, Tuple, Union
+
+from typing import Any, AsyncGenerator, List, Optional, Tuple, Union
 
 from libsimba.schemas import (
     ConnectionConfig,
+    FieldFilter,
     FileDict,
+    FilterOp,
     Login,
     MethodCallArgs,
     SearchFilter,
     TxnHeaders,
 )
 from libsimba.simba_contract import SimbaContract
-from libsimba.simba_request import GetRequest, PostRequest, PutRequest, SimbaRequest
+from libsimba.simba_request import GetRequest, PatchRequest, PostRequest, PutRequest, SimbaRequest
 from libsimba.simba_sync import SimbaSync
-from libsimba.utils import Path, get_address, get_deployed_artifact_id
+from libsimba.utils import Path, get_address, get_deployed_artifact_id, get_address_by_name
 
 
 logger = logging.getLogger(__name__)
@@ -186,9 +188,9 @@ class Simba(SimbaSync):
         login: Login = None,
         config: ConnectionConfig = None,
     ) -> dict:
-        return await GetRequest(
-            endpoint=Path.APP.format(org, app_id), login=login
-        ).get(config=config)
+        return await GetRequest(endpoint=Path.APP.format(org, app_id), login=login).get(
+            config=config
+        )
 
     async def list_application_transactions(
         self,
@@ -351,13 +353,14 @@ class Simba(SimbaSync):
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
     ) -> AsyncGenerator[List[dict], None]:
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve_iter(config=config)
@@ -366,13 +369,14 @@ class Simba(SimbaSync):
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
     ) -> List[dict]:
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return await SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve(config=config)
@@ -465,13 +469,14 @@ class Simba(SimbaSync):
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
     ) -> List[dict]:
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return await SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve(config=config)
@@ -480,13 +485,14 @@ class Simba(SimbaSync):
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
     ) -> AsyncGenerator[List[dict], None]:
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve_iter(config=config)
@@ -554,15 +560,15 @@ class Simba(SimbaSync):
         login: Login = None,
         config: ConnectionConfig = None,
     ) -> dict:
-        return await PostRequest(
+        return await PatchRequest(
             endpoint=Path.APP_TXN.format(app_id, txn_id), login=login
-        ).post(json_payload={"transaction": txn}, config=config)
+        ).patch(json_payload={"transaction": txn}, config=config)
 
     async def save_design(
         self,
         org: str,
         name: str,
-        code: str,
+        code: Union[str, dict],
         design_id: Optional[str] = None,
         target_contract: str = None,
         libraries: dict = None,
@@ -573,7 +579,13 @@ class Simba(SimbaSync):
         config: ConnectionConfig = None,
     ) -> dict:
         if encode:
-            code = base64.b64encode(code.encode()).decode("utf-8")
+            if isinstance(code, dict):
+                tmp = {}
+                for k, v in code.items():
+                    tmp[k] = base64.b64encode(v.encode()).decode("utf-8")
+                code = tmp
+            else:
+                code = base64.b64encode(code.encode()).decode("utf-8")
         full = {"name": name, "code": code, "language": "solidity"}
         if target_contract:
             full["target_contract"] = target_contract
@@ -614,7 +626,7 @@ class Simba(SimbaSync):
         else:
             if total_time > max_time:
                 raise ValueError("[wait_for_deployment] :: waited way too long")
-            time.sleep(2)
+            await asyncio.sleep(2)
             total_time += 2
             return await self.wait_for_deployment(
                 org, uid, total_time=total_time, login=login, config=config
@@ -675,6 +687,68 @@ class Simba(SimbaSync):
             login=login,
         ).post(json_payload=full, config=config)
 
+    async def deploy_library(
+        self,
+        org: str,
+        lib_name: str,
+        blockchain: str,
+        code: str,
+        encode: bool = True,
+        app_name=None,
+        args=None,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        if encode:
+            code = base64.b64encode(code.encode()).decode("utf-8")
+        full = {
+            "blockchain": blockchain,
+            "language": "solidity",
+            "code": code,
+            "lib_name": lib_name,
+        }
+        if app_name:
+            full["app_name"] = app_name
+        if args:
+            full["args"] = args
+        return await PostRequest(
+            endpoint=Path.LIBRARIES.create(org),
+            login=login,
+        ).post(json_payload=full, config=config)
+
+    async def wait_for_deploy_library(
+        self,
+        org: str,
+        lib_name: str,
+        blockchain: str,
+        code: str,
+        app_name=None,
+        args=None,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> Tuple[str, str]:
+
+        res = await self.deploy_library(
+            org=org,
+            lib_name=lib_name,
+            blockchain=blockchain,
+            code=code,
+            args=args,
+            app_name=app_name,
+            login=login,
+            config=config
+        )
+        deployment_id = res["deployment_id"]
+        try:
+            deployed = await self.wait_for_deployment(org, deployment_id)
+            address = get_address_by_name(deployed, lib_name)
+            library_id = get_deployed_artifact_id(deployed)
+        except Exception as ex:
+            logger.warning("[wait_for_deploy_library] :: failed to wait for deployment: {}".format(ex))
+            address = None
+            library_id = None
+        return address, library_id
+
     async def wait_for_deploy_design(
         self,
         org: str,
@@ -762,7 +836,7 @@ class Simba(SimbaSync):
         else:
             if total_time > max_time:
                 raise ValueError("waited way too long")
-            time.sleep(2)
+            await asyncio.sleep(2)
             total_time += 2
             return await self.wait_for_org_transaction(
                 org=org,
@@ -806,6 +880,118 @@ class Simba(SimbaSync):
             login=login,
         ).retrieve(config=config)
 
+    async def get_abi(
+        self,
+        blockchain: str,
+        contract_address: str,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        return await SimbaRequest(
+            endpoint=Path.CONTRACT_ABI.create(blockchain, contract_address),
+            login=login,
+        ).send(config=config)
+
+    async def get_accounts(
+        self,
+        nickname: Optional[str] = None,
+        alias: Optional[str] = None,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> List[dict]:
+        params = None
+        if nickname or alias:
+            params = SearchFilter()
+            if nickname:
+                params.add_filter(FieldFilter(field="nickname", op=FilterOp.EQ, value=nickname))
+            if alias:
+                params.add_filter(FieldFilter(field="alias", op=FilterOp.EQ, value=alias))
+
+        return await SimbaRequest(
+            endpoint=Path.USER_ACCOUNTS,
+            query_params=params,
+            login=login,
+        ).retrieve(config=config)
+
+    async def get_account(
+            self,
+            uid: str,
+            login: Login = None,
+            config: ConnectionConfig = None,
+    ) -> dict:
+        return await SimbaRequest(
+            endpoint=Path.USER_ACCOUNT.format(uid),
+            login=login,
+        ).send(config=config)
+
+    async def create_account(
+        self,
+        network_subtype: str,
+        network: str,
+        nickname: str,
+        alias: str,
+        network_type: Optional[str] = "ethereum",
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        payload = {
+          "network_type": network_type,
+          "network_subtype": network_subtype,
+          "network": network,
+          "nickname": nickname,
+          "alias": alias
+        }
+        return await SimbaRequest(
+            method="POST",
+            endpoint=Path.USER_ACCOUNTS,
+            login=login,
+        ).send(config=config, json_payload=payload)
+
+    async def set_account(
+        self,
+        network_subtype: str,
+        network: str,
+        nickname: str,
+        alias: str,
+        address: str,
+        private_key: str,
+        network_type: Optional[str] = "ethereum",
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        payload = {
+          "network_type": network_type,
+          "network_subtype": network_subtype,
+          "network": network,
+          "nickname": nickname,
+          "alias": alias,
+          "public_key": address,
+          "private_key": private_key
+        }
+        return await SimbaRequest(
+            method="POST",
+            endpoint=Path.USER_ACCOUNT_SET,
+            login=login,
+        ).send(config=config, json_payload=payload)
+
+    async def account_sign(
+        self,
+        uid: str,
+        input_pairs: List[Tuple[str, Any]],
+        hash_message: Optional[bool] = False,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        payload = {
+            "input_pairs": [list(t) for t in input_pairs],
+            "hash_message": hash_message
+        }
+        return await SimbaRequest(
+            method="POST",
+            endpoint=Path.USER_ACCOUNT_SIGN.format(uid),
+            login=login,
+        ).send(config=config, json_payload=payload)
+
     async def get_artifacts(
         self,
         org: str,
