@@ -1,4 +1,4 @@
-#  Copyright (c) 2023 SIMBA Chain Inc. https://simbachain.com
+#  Copyright (c) 2024 SIMBA Chain Inc. https://simbachain.com
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -22,11 +22,13 @@ import base64
 import logging
 import time
 
-from typing import Generator, List, Optional, Tuple, Union
+from typing import Any, Generator, List, Optional, Tuple, Union
 
 from libsimba.config import settings
 from libsimba.schemas import (
     ConnectionConfig,
+    FieldFilter,
+    FilterOp,
     FileDict,
     Login,
     MethodCallArgs,
@@ -34,8 +36,8 @@ from libsimba.schemas import (
     TxnHeaders,
 )
 from libsimba.simba_contract_sync import SimbaContractSync
-from libsimba.simba_request import GetRequest, PostRequest, PutRequest, SimbaRequest
-from libsimba.utils import Path, get_address, get_deployed_artifact_id
+from libsimba.simba_request import GetRequest, PatchRequest, PostRequest, PutRequest, SimbaRequest
+from libsimba.utils import Path, get_address, get_deployed_artifact_id, get_address_by_name
 
 
 logger = logging.getLogger(__name__)
@@ -52,7 +54,9 @@ class SimbaSync:
         """
         settings(**kwargs)
 
-    def smart_contract_client(self, app_name: str, contract_name: str) -> SimbaContractSync:
+    def smart_contract_client(
+        self, app_name: str, contract_name: str
+    ) -> SimbaContractSync:
         return SimbaContractSync(self, app_name, contract_name)
 
     def whoami(self, login: Login = None, config: ConnectionConfig = None) -> dict:
@@ -203,7 +207,7 @@ class SimbaSync:
         GET ``/user/wallet/``
 
         Get current user's wallet
-            
+
         :Keyword Arguments:
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
@@ -376,7 +380,7 @@ class SimbaSync:
 
         :param app_id: Application id or name
         :type app_id: str
-            
+
         :Keyword Arguments:
             * **query_args** (`Optional[SearchFilter]`)
             * **login** (`Optional[Login]`)
@@ -411,9 +415,9 @@ class SimbaSync:
         :return: Application information
         :rtype: dict
         """
-        return GetRequest(
-            endpoint=Path.APP.format(org, app_id), login=login
-        ).get_sync(config=config)
+        return GetRequest(endpoint=Path.APP.format(org, app_id), login=login).get_sync(
+            config=config
+        )
 
     def list_application_transactions(
         self,
@@ -787,7 +791,7 @@ class SimbaSync:
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
@@ -801,18 +805,18 @@ class SimbaSync:
         :type app_id: str
         :param contract_name: Contract API name
         :type contract_name: str
-        :param event_name: Name of the Event
-        :type event_name: str
 
         :Keyword Arguments:
+            * **event_name** (`Optional[str]`)
             * **query_args** (`Optional[SearchFilter]`)
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
         :return: A generator of events
         :rtype: Generator[List[dict], None, None]
         """
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve_iter_sync(config=config)
@@ -821,7 +825,7 @@ class SimbaSync:
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
@@ -835,18 +839,18 @@ class SimbaSync:
         :type app_id: str
         :param contract_name: Contract API name
         :type contract_name: str
-        :param event_name: Name of the Event
-        :type event_name: str
 
         :Keyword Arguments:
+            * **event_name** (`Optional[str]`)
             * **query_args** (`Optional[SearchFilter]`)
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
         :return: A list of events
         :rtype: List[dict]
         """
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve_sync(config=config)
@@ -901,7 +905,7 @@ class SimbaSync:
         :type contract_name: str
         :param transaction_hash: The hash of the receipt
         :type transaction_hash: str
-            
+
         :Keyword Arguments:
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
@@ -1047,7 +1051,7 @@ class SimbaSync:
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
@@ -1061,18 +1065,18 @@ class SimbaSync:
         :type app_id: str
         :param contract_name: Contract API name
         :type contract_name: str
-        :param event_name: Event name
-        :type event_name: str
 
         :Keyword Arguments:
+            * **event_name** (`Optional[str]`)
             * **query_args** (`Optional[SearchFilter]`)
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
         :return: A list of transactions
         :rtype: List[dict]
         """
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve_sync(config=config)
@@ -1081,7 +1085,7 @@ class SimbaSync:
         self,
         app_id: str,
         contract_name: str,
-        event_name: str,
+        event_name: Optional[str] = None,
         query_args: Optional[SearchFilter] = None,
         login: Login = None,
         config: ConnectionConfig = None,
@@ -1095,18 +1099,18 @@ class SimbaSync:
         :type app_id: str
         :param contract_name: Contract API name
         :type contract_name: str
-        :param event_name: Event name
-        :type event_name: str
 
         :Keyword Arguments:
+            * **event_name** (`Optional[str]`)
             * **query_args** (`Optional[SearchFilter]`)
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
         :return: A generator of transactions
         :rtype: Generator[List[dict], None, None]
         """
+        query_args = self.add_event_name(event_name=event_name, query_args=query_args)
         return SimbaRequest(
-            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name, event_name),
+            endpoint=Path.CONTRACT_EVENTS.format(app_id, contract_name),
             query_params=query_args,
             login=login,
         ).retrieve_iter_sync(config=config)
@@ -1239,7 +1243,7 @@ class SimbaSync:
         config: ConnectionConfig = None,
     ):
         """
-        POST ``/v2/apps/{application}/transactions/{identifier}/``
+        PATCH ``/v2/apps/{application}/transactions/{identifier}/``
 
         Submit a signed transaction to a contract method.
 
@@ -1249,22 +1253,22 @@ class SimbaSync:
         :type txn_id: str
         :param txn: The signed transaction keyed to the ``transaction`` value.
         :type txn: dict
-            
+
         :Keyword Arguments:
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
         :return: The transaction
         :rtype: dict
         """
-        return PostRequest(
+        return PatchRequest(
             endpoint=Path.APP_TXN.format(app_id, txn_id), login=login
-        ).post_sync(json_payload={"transaction": txn}, config=config)
+        ).patch_sync(json_payload={"transaction": txn}, config=config)
 
     def save_design(
         self,
         org: str,
         name: str,
-        code: str,
+        code: Union[str, dict],
         design_id: Optional[str] = None,
         target_contract: str = None,
         libraries: dict = None,
@@ -1285,7 +1289,7 @@ class SimbaSync:
         :param name: The name of the design.
         :type name: str
         :param code: the code
-        :type code: str
+        :type code: Union[str, dict]
 
         :Keyword Arguments:
             * **design_id** (`str`) - A design ID. If provided, this will update the given design via a PUT. Otherwise a new design will be created.
@@ -1300,7 +1304,13 @@ class SimbaSync:
         :rtype: dict
         """
         if encode:
-            code = base64.b64encode(code.encode()).decode("utf-8")
+            if isinstance(code, dict):
+                tmp = {}
+                for k, v in code.items():
+                    tmp[k] = base64.b64encode(v.encode()).decode("utf-8")
+                code = tmp
+            else:
+                code = base64.b64encode(code.encode()).decode("utf-8")
         full = {"name": name, "code": code, "language": "solidity"}
         if target_contract:
             full["target_contract"] = target_contract
@@ -1328,7 +1338,7 @@ class SimbaSync:
         org: str,
         uid: str,
         total_time: int = 0,
-        max_time: int = 480,
+        max_time: int = 960,
         login: Login = None,
         config: ConnectionConfig = None,
     ) -> dict:
@@ -1442,7 +1452,7 @@ class SimbaSync:
         :param org: The organisation to deploy to.
         :type org: str
         :param app: The app to deploy to.
-        :type org: str
+        :type app: str
         :param api_name: The api name of the contract.
         :type api_name: str
         :param artifact_id: The id of the artifact to deploy
@@ -1475,6 +1485,92 @@ class SimbaSync:
             login=login,
         ).post_sync(json_payload=full, config=config)
 
+    def deploy_library(
+        self,
+        org: str,
+        lib_name: str,
+        blockchain: str,
+        code: str,
+        encode: bool = True,
+        app_name=None,
+        args=None,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        """
+        POST ``/v2/organisations/{organisation}/deployments/library/``
+
+        Deploy a library. This results in a deployment object being created and returned.
+
+        :param org: The organisation to deploy to.
+        :type org: str
+        :param code: The library code.
+        :type code: str
+        :param lib_name: The name of the library.
+        :type lib_name: str
+        :param blockchain: The name of the blockchain to deploy to.
+        :type blockchain: str
+        :param encode: whether to base64 encode the code or not.
+        :type encode: bool
+
+        :Keyword Arguments:
+            * **app_name** (`str`) - App name for the library.
+            * **args** (`dict`) - Constructor args for the deployment.
+            * **login** (`Optional[Login]`)
+            * **config** (`Optional[ConnectionConfig]`)
+        :return: A deployment object
+        :rtype: Deployment
+        """
+        if encode:
+            code = base64.b64encode(code.encode()).decode("utf-8")
+        full = {
+            "blockchain": blockchain,
+            "language": "solidity",
+            "code": code,
+            "lib_name": lib_name,
+        }
+        if app_name:
+            full["app_name"] = app_name
+        if args:
+            full["args"] = args
+        return PostRequest(
+            endpoint=Path.LIBRARIES.create(org),
+            login=login,
+        ).post_sync(json_payload=full, config=config)
+
+    def wait_for_deploy_library(
+        self,
+        org: str,
+        lib_name: str,
+        blockchain: str,
+        code: str,
+        app_name=None,
+        args=None,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> Tuple[str, str]:
+
+        res = self.deploy_library(
+            org=org,
+            lib_name=lib_name,
+            blockchain=blockchain,
+            code=code,
+            args=args,
+            app_name=app_name,
+            login=login,
+            config=config
+        )
+        deployment_id = res["deployment_id"]
+        try:
+            deployed = self.wait_for_deployment(org, deployment_id)
+            address = get_address_by_name(deployed, lib_name)
+            library_id = get_deployed_artifact_id(deployed)
+        except Exception as ex:
+            logger.warning("[wait_for_deploy_library] :: failed to wait for deployment: {}".format(ex))
+            address = None
+            library_id = None
+        return address, library_id
+
     def wait_for_deploy_design(
         self,
         org: str,
@@ -1604,7 +1700,7 @@ class SimbaSync:
 
         :param org: The organisation.
         :type org: str
-            
+
         :Keyword Arguments:
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
@@ -1736,7 +1832,7 @@ class SimbaSync:
 
         :param org: The organisation.
         :type org: str
-            
+
         :Keyword Arguments:
             * **login** (`Optional[Login]`)
             * **config** (`Optional[ConnectionConfig]`)
@@ -1748,6 +1844,192 @@ class SimbaSync:
             login=login,
         ).retrieve_sync(config=config)
 
+    def get_abi(
+        self,
+        blockchain: str,
+        contract_address: str,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        """
+        GET ``/services/contracts/abi/{contract_address}/``
+
+        Get the ABI and metadata for a contract address.
+
+        :param contract_address: The address of the deployed contract.
+        :type contract_address: str
+        :param blockchain: The name of the blockchain the contract is deployed on.
+        :type blockchain: str
+
+        :Keyword Arguments:
+            * **login** (`Optional[Login]`)
+            * **config** (`Optional[ConnectionConfig]`)
+        :return: a Dict with keys `abi` and `metadata`.
+        :rtype: dict
+        """
+        return SimbaRequest(
+            endpoint=Path.CONTRACT_ABI.create(blockchain, contract_address),
+            login=login,
+        ).send_sync(config=config)
+
+    def get_accounts(
+        self,
+        nickname: Optional[str] = None,
+        alias: Optional[str] = None,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> List[dict]:
+        """
+        GET ``/user/accounts/``
+
+        Get the accounts for the current user. Optionally filter on
+        nickname or alias.
+
+        :Keyword Arguments:
+            * **nickname** (`Optional[str]`)
+            * **alias** (`Optional[str]`)
+            * **login** (`Optional[Login]`)
+            * **config** (`Optional[ConnectionConfig]`)
+        :return: a Dict with keys `abi` and `metadata`.
+        :rtype: dict
+        """
+        params = None
+        if nickname or alias:
+            params = SearchFilter()
+            if nickname:
+                params.add_filter(FieldFilter(field="nickname", op=FilterOp.EQ, value=nickname))
+            if alias:
+                params.add_filter(FieldFilter(field="alias", op=FilterOp.EQ, value=alias))
+
+        return SimbaRequest(
+            endpoint=Path.USER_ACCOUNTS,
+            query_params=params,
+            login=login,
+        ).retrieve_sync(config=config)
+
+    def get_account(
+        self,
+        uid: str,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        """
+        GET ``/user/accounts/{id}``
+
+        Get the account for the current user with the given id.
+
+        :Keyword Arguments:
+            * **login** (`Optional[Login]`)
+            * **config** (`Optional[ConnectionConfig]`)
+        :return: a Dict with keys `abi` and `metadata`.
+        :rtype: dict
+        """
+        return SimbaRequest(
+            endpoint=Path.USER_ACCOUNT.format(uid),
+            login=login,
+        ).send_sync(config=config)
+
+    def create_account(
+        self,
+        network_subtype: str,
+        network: str,
+        nickname: str,
+        alias: str,
+        network_type: Optional[str] = "ethereum",
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        """
+        POST ``/user/accounts/``
+
+        Create a new account for the current user.
+
+        :Keyword Arguments:
+            * **login** (`Optional[Login]`)
+            * **config** (`Optional[ConnectionConfig]`)
+        :return: a Dict with keys `abi` and `metadata`.
+        :rtype: dict
+        """
+        payload = {
+          "network_type": network_type,
+          "network_subtype": network_subtype,
+          "network": network,
+          "nickname": nickname,
+          "alias": alias
+        }
+        return SimbaRequest(
+            method="POST",
+            endpoint=Path.USER_ACCOUNTS,
+            login=login,
+        ).send_sync(config=config, json_payload=payload)
+
+    def set_account(
+        self,
+        network_subtype: str,
+        network: str,
+        nickname: str,
+        alias: str,
+        address: str,
+        private_key: str,
+        network_type: Optional[str] = "ethereum",
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        """
+        POST ``/user/accounts/set/``
+
+        Create a new account for the current user.
+
+        :Keyword Arguments:
+            * **login** (`Optional[Login]`)
+            * **config** (`Optional[ConnectionConfig]`)
+        :return: a Dict with keys `abi` and `metadata`.
+        :rtype: dict
+        """
+        payload = {
+          "network_type": network_type,
+          "network_subtype": network_subtype,
+          "network": network,
+          "nickname": nickname,
+          "alias": alias,
+          "public_key": address,
+          "private_key": private_key
+        }
+        return SimbaRequest(
+            method="POST",
+            endpoint=Path.USER_ACCOUNT_SET,
+            login=login,
+        ).send_sync(config=config, json_payload=payload)
+
+    def account_sign(
+        self,
+        uid: str,
+        input_pairs: List[Tuple[str, Any]],
+        hash_message: Optional[bool] = False,
+        login: Login = None,
+        config: ConnectionConfig = None,
+    ) -> dict:
+        """
+        POST ``/user/accounts/{id}``
+
+        Get the account for the current user to sign inputs.
+
+        :Keyword Arguments:
+            * **login** (`Optional[Login]`)
+            * **config** (`Optional[ConnectionConfig]`)
+        :return: a Dict with keys `abi` and `metadata`.
+        :rtype: dict
+        """
+        payload = {
+            "input_pairs": [list(t) for t in input_pairs],
+            "hash_message": hash_message
+        }
+        return SimbaRequest(
+            method="POST",
+            endpoint=Path.USER_ACCOUNT_SIGN.format(uid),
+            login=login,
+        ).send_sync(config=config, json_payload=payload)
+
     def subscribe(
         self,
         org: str,
@@ -1909,3 +2191,15 @@ class SimbaSync:
                 login=login,
                 config=config,
             )
+
+    def add_event_name(
+        self,
+        event_name: Optional[str] = None,
+        query_args: Optional[SearchFilter] = None
+    ) -> Optional[SearchFilter]:
+        if event_name:
+            if not query_args:
+                query_args = SearchFilter()
+            if not query_args.has_filter_value(field="event_name", op=FilterOp.EQ, value=event_name):
+                query_args.add_filter(FieldFilter(op=FilterOp.EQ, field="event_name", value=event_name))
+        return query_args
\ No newline at end of file
